//YWROBOT
#ifndef ZUNO_LCD_I2C_h
#define ZUNO_LCD_I2C_h

#include "Arduino.h"
#include "Print.h" 
#include "Wire.h"
#include "binary.h"

// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00

// flags for backlight control
#define LCD_BACKLIGHT 0x08
#define LCD_NOBACKLIGHT 0x00

#define En B00000100  // Enable bit
#define Rw B00000010  // Read/Write bit
#define Rs B00000001  // Register select bit

class ZUNO_LCD_I2C : public Print {
public:
  ZUNO_LCD_I2C(byte lcd_Addr,byte lcd_cols,byte lcd_rows);
  void begin(byte cols, byte rows, byte charsize = LCD_5x8DOTS );
  void clear();
  void home();
  void noDisplay();
  void display();
  void noBlink();
  void blink();
  void noCursor();
  void cursor();
  void scrollDisplayLeft();
  void scrollDisplayRight();
  void printLeft();
  void printRight();
  void leftToRight();
  void rightToLeft();
  void shiftIncrement();
  void shiftDecrement();
  void noBacklight();
  void backlight();
  void autoscroll();
  void bottomline();
  void noAutoscroll(); 
  void createChar(byte, byte *);
  void setCursor(byte, byte); 
  virtual size_t write(byte);
  void command(byte);
  void init();

////compatibility API function aliases
void blink_on();						// alias for blink()
void blink_off();       					// alias for noBlink()
void cursor_on();      	 					// alias for cursor()
void cursor_off();      					// alias for noCursor()
void setBacklight(byte new_val);				// alias for backlight() and nobacklight()
void load_custom_character(byte char_num, byte *rows);	// alias for createChar()
void printstr(const char *);

////Unsupported API functions (not implemented in this library)
byte status();
void setContrast(byte new_val);
byte keypad();
void setDelay(int,int);
void on();
void off();
byte init_bargraph(byte graphtype);
void draw_horizontal_graph(byte row, byte column, byte len,  byte pixel_col_end);
void draw_vertical_graph(byte row, byte column, byte len,  byte pixel_col_end);
	 

private:
  void init_priv();
  void send(byte, byte);
  void write4bits(byte);
  void expanderWrite(byte);
  void pulseEnable(byte);
  byte _Addr;
  byte _displayfunction;
  byte _displaycontrol;
  byte _displaymode;
  byte _numlines;
  byte _cols;
  byte _rows;
  byte _backlightval;
};

#endif
